package tests;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import com.google.common.eventbus.EventBus;

import events.MyEvent;
import listeners.DeadEventListener;
import listeners.IntegerListener;
import listeners.MyEventListener;
import listeners.MyEventListenerClone;
import listeners.NumberListener;
import listeners.MultipleListener;

public class EventBusTest {

	@Test
	public void shouldReceiveEvent() throws Exception {

		// given
		EventBus eventBus = new EventBus("test");
		MyEventListener listener = new MyEventListener();

		eventBus.register(listener);

		// when
		eventBus.post(new MyEvent(200));

		// then
		assertTrue(listener.getLastMessage() == 200);
	}

	@Test
	public void shouldReceiveMultipleEvents() throws Exception {

		// given
		EventBus eventBus = new EventBus("test");
		MultipleListener multiListener = new MultipleListener();

		eventBus.register(multiListener);

		// when
		eventBus.post(new Integer(100));
		eventBus.post(new Long(800));

		// then
		assertTrue(multiListener.getLastInteger() == 100);
		assertTrue(multiListener.getLastLong() == 800L);
	}
	
	@Test
	public void shouldDetectEventWithoutListeners() throws Exception {
	 
	    // given
	    EventBus eventBus = new EventBus("test");
	    DeadEventListener deadEventListener = new DeadEventListener();
	    
	    eventBus.register(deadEventListener);
	 
	    // when
	    eventBus.post(new MyEvent(200));
	 
	    assertTrue(deadEventListener.isNotDelivered());
	}
	
	@Test
	public void shouldGetEventsFromSubclass() throws Exception {
	  // given
	  EventBus eventBus = new EventBus("test");
	  IntegerListener integerListener = new IntegerListener();
	  NumberListener numberListener = new NumberListener();
	  eventBus.register(integerListener);
	  eventBus.register(numberListener);
	 
	  // when
	  eventBus.post(new Integer(100));
	 
	  // then
	  assertTrue(integerListener.getLastMessage() == 100);
	  assertTrue(numberListener.getLastMessage().intValue() == 100);
	 
	  //when
	  eventBus.post(new Long(200L));
	 
	  // then
	  // this one should has the old value as it listens only for Integers
	  assertTrue(integerListener.getLastMessage() == 100);
	  assertTrue(numberListener.getLastMessage().longValue() == 200L);
	}
	
	@Test
	public void shouldReceiveEventDouble() throws Exception {

		// given
		EventBus eventBus = new EventBus("test");
		MyEventListener listener = new MyEventListener();
		MyEventListenerClone listener2 = new MyEventListenerClone();

		eventBus.register(listener);
		eventBus.register(listener2);

		// when
		eventBus.post(new MyEvent(200));
		eventBus.post(new MyEvent(200));

		// then
		assertTrue(listener.getLastMessage() == 200);
		assertTrue(listener2.getLastMessage() == 200);
	}

}
